@@ -51,8 +51,11 @@ def generate_doc(generator, help_command):
         for arg_name in help_command.arg_table:
             if _is_argument_hidden(help_command.arg_table[arg_name]):
                 continue
+            generator.doc_option_start(arg_name, help_command)
             generator.doc_option(arg_name, help_command)
             generator.doc_option_example(arg_name, help_command)
+            generator.doc_option_form_factors(arg_name, help_command)
+            generator.doc_option_end(arg_name, help_command)
     generator.doc_options_end(help_command)
     generator.doc_subitems_start(help_command)
     if help_command.command_table:
@@ -64,6 +67,7 @@ def generate_doc(generator, help_command):
     generator.doc_subitems_end(help_command)
     generator.doc_examples(help_command)
     generator.doc_output(help_command)
+    generator.doc_form_factors(help_command)
     generator.doc_relateditems_start(help_command)
     if help_command.related_items:
         for related_item in sorted(help_command.related_items):
@@ -145,39 +149,26 @@ class CLIDocumentGenerator(object):
         self._documented_arg_groups = []
 
     def doc_options_start(self, help_command):
-        doc = help_command.doc
-        doc.style.h2('Options')
-        if not help_command.arg_table:
-            doc.write('*None*\n')
+        pass
 
-    def doc_options_end(self, help_command):
+    def doc_option_start(self, arg_name, help_command):
         pass
 
     def doc_option(self, arg_name, help_command):
-        doc = help_command.doc
-        argument = help_command.arg_table[arg_name]
-        if _is_argument_hidden(argument):
-            return
-        if argument.group_name in self._arg_groups:
-            if argument.group_name in self._documented_arg_groups:
-                # This arg is already documented so we can move on.
-                return
-            name = ' | '.join(
-                ['``%s``' % a.cli_name for a in
-                 self._arg_groups[argument.group_name]])
-            self._documented_arg_groups.append(argument.group_name)
-        else:
-            name = '``%s``' % argument.cli_name
-        doc.write('%s (%s)\n' % (name, argument.cli_type_name))
-        doc.style.indent()
-        doc.include_doc_string(argument.documentation)
-        self._document_enums(argument, doc)
-        doc.style.dedent()
-        doc.style.new_paragraph()
+        pass
 
     def doc_option_example(self, arg_name, help_command):
         pass
 
+    def doc_option_form_factors(self, arg_name, help_command):
+        pass
+
+    def doc_option_end(self, arg_name, help_command):
+        pass
+
+    def doc_options_end(self, help_command):
+        pass
+
     def doc_relateditems_start(self, help_command):
         if help_command.related_items:
             doc = help_command.doc
@@ -192,23 +183,6 @@ class CLIDocumentGenerator(object):
         )
         doc.write('\n')
 
-    def _document_enums(self, argument, doc):
-        if hasattr(argument, 'argument_model'):
-            model = argument.argument_model
-            if isinstance(model, StringShape):
-                if model.enum:
-                    self._write_possible_values(doc, model.enum)
-                if model.supported_options:
-                    self._write_possible_values(doc, model.supported_options)
-
-    def _write_possible_values(self, doc, possible_values):
-        doc.style.new_paragraph()
-        doc.write('Possible values:')
-        doc.style.start_ul()
-        for value in possible_values:
-            doc.style.li('``%s``' % value)
-        doc.style.end_ul()
-
     def doc_subitems_start(self, help_command):
         pass
 
@@ -221,6 +195,9 @@ class CLIDocumentGenerator(object):
     def doc_output(self, help_command):
         pass
 
+    def doc_form_factors(self, help_command):
+        pass
+
 
 class ProviderDocumentGenerator(CLIDocumentGenerator):
 
@@ -241,6 +218,9 @@ class ProviderDocumentGenerator(CLIDocumentGenerator):
         doc = help_command.doc
         doc.style.h2('Options')
 
+    def doc_option_start(self, arg_name, help_command):
+        pass
+
     def doc_option(self, arg_name, help_command):
         doc = help_command.doc
         argument = help_command.arg_table[arg_name]
@@ -253,6 +233,18 @@ class ProviderDocumentGenerator(CLIDocumentGenerator):
                 doc.style.li(choice)
             doc.style.end_ul()
 
+    def doc_option_example(self, arg_name, help_command):
+        pass
+
+    def doc_option_form_factors(self, arg_name, help_command):
+        pass
+
+    def doc_option_end(self, arg_name, help_command):
+        pass
+
+    def doc_options_end(self, help_command):
+        pass
+
     def doc_subitems_start(self, help_command):
         doc = help_command.doc
         doc.style.h2('Available Commands')
@@ -287,12 +279,21 @@ class ServiceDocumentGenerator(CLIDocumentGenerator):
     def doc_options_start(self, help_command):
         pass
 
+    def doc_option_start(self, arg_name, help_command):
+        pass
+
     def doc_option(self, arg_name, help_command):
         pass
 
     def doc_option_example(self, arg_name, help_command):
         pass
 
+    def doc_option_form_factors(self, arg_name, help_command):
+        pass
+
+    def doc_option_end(self, arg_name, help_command):
+        pass
+
     def doc_options_end(self, help_command):
         pass
 
@@ -491,6 +492,41 @@ class OperationDocumentGenerator(CLIDocumentGenerator):
                 self._json_example(doc, member_model, stack)
                 need_comma = True
 
+    def doc_options_start(self, help_command):
+        doc = help_command.doc
+        doc.style.h2('Options')
+        if not help_command.arg_table:
+            doc.write('*None*\n')
+
+    def doc_option_start(self, arg_name, help_command):
+        pass
+
+    def doc_option(self, arg_name, help_command):
+        doc = help_command.doc
+        argument = help_command.arg_table[arg_name]
+        if _is_argument_hidden(argument):
+            return
+        if argument.group_name in self._arg_groups:
+            if argument.group_name in self._documented_arg_groups:
+                # This arg is already documented so we can move on.
+                return
+            name = ' | '.join(
+                ['``%s``' % a.cli_name for a in
+                 self._arg_groups[argument.group_name]])
+        else:
+            name = '``%s``' % argument.cli_name
+        doc.write('%s (%s)\n' % (name, argument.cli_type_name))
+        doc.style.indent()
+        doc.style.new_paragraph()
+        doc.include_doc_string(argument.documentation)
+        self._document_enums(argument, doc)
+        if hasattr(argument.argument_model, 'members'):
+            doc.style.new_paragraph()
+            for member_name, member_shape in argument.argument_model.members.items():
+                self.doc_member(doc, member_name, member_shape, stack=[])
+        doc.style.dedent()
+        doc.style.new_paragraph()
+
     def doc_option_example(self, arg_name, help_command):
         doc = help_command.doc
         cli_argument = help_command.arg_table[arg_name]
@@ -549,12 +585,28 @@ class OperationDocumentGenerator(CLIDocumentGenerator):
             doc.style.end_codeblock()
             doc.style.new_paragraph()
 
-    def _write_possible_values(self, doc, possible_values):
-        doc.style.new_paragraph()
-        doc.write("Possible values:\n")
-        for value in possible_values:
-            doc.write("    %s\n" % value)
-        doc.write("\n")
+    def doc_option_form_factors(self, arg_name, help_command):
+        doc = help_command.doc
+        argument = help_command.arg_table[arg_name]
+        if _is_argument_hidden(argument):
+            return
+        if argument.group_name in self._arg_groups:
+            if argument.group_name in self._documented_arg_groups:
+                # This arg is already documented so we can move on.
+                return
+        if argument.argument_model.form_factors is not None:
+            self._write_form_factors(doc, argument.argument_model.form_factors)
+
+    def doc_option_end(self, arg_name, help_command):
+        argument = help_command.arg_table[arg_name]
+        if argument.group_name in self._arg_groups:
+            if argument.group_name in self._documented_arg_groups:
+                # This arg is already documented so we can move on.
+                return
+            self._documented_arg_groups.append(argument.group_name)
+
+    def doc_options_end(self, help_command):
+        pass
 
     def doc_output(self, help_command):
         doc = help_command.doc
@@ -565,44 +617,7 @@ class OperationDocumentGenerator(CLIDocumentGenerator):
             doc.write('None')
         else:
             for member_name, member_shape in output_shape.members.items():
-                self._doc_member_for_output(doc, member_name, member_shape, stack=[])
-
-    def _doc_member_for_output(self, doc, member_name, member_shape, stack):
-        if member_shape.name in stack:
-            # Document the recursion once, otherwise just
-            # note the fact that it's recursive and return.
-            if stack.count(member_shape.name) > 1:
-                if member_shape.type_name == OBJECT_TYPE:
-                    doc.write('( ... recursive ... )')
-                return
-        stack.append(member_shape.name)
-        try:
-            self._do_doc_member_for_output(doc, member_name,
-                                           member_shape, stack)
-        finally:
-            stack.pop()
-
-    def _do_doc_member_for_output(self, doc, member_name, member_shape, stack):
-        docs = member_shape.documentation
-        if member_name:
-            doc.write('%s -> (%s)' % (member_name, member_shape.type_name))
-        else:
-            doc.write('(%s)' % member_shape.type_name)
-        doc.style.indent()
-        doc.style.new_paragraph()
-        doc.include_doc_string(docs)
-        doc.style.new_paragraph()
-        member_type_name = member_shape.type_name
-        if member_type_name == OBJECT_TYPE:
-            for sub_name, sub_shape in member_shape.members.items():
-                self._doc_member_for_output(doc, sub_name, sub_shape, stack)
-        elif member_type_name == LIST_TYPE:
-            self._doc_member_for_output(doc, '', member_shape.member, stack)
-        elif member_type_name == MAP_TYPE:
-            self._doc_member_for_output(doc, 'key', member_shape.key, stack)
-            self._doc_member_for_output(doc, 'value', member_shape.value, stack)
-        doc.style.dedent()
-        doc.style.new_paragraph()
+                self.doc_member(doc, member_name, member_shape, stack=[])
 
     def doc_examples(self, help_command):
         operation = help_command.command_lineage.replace('.', os.path.sep)
@@ -614,3 +629,78 @@ class OperationDocumentGenerator(CLIDocumentGenerator):
             fp = open(doc_path)
             for line in fp.readlines():
                 help_command.doc.write(line)
+
+    def doc_form_factors(self, help_command):
+        operation_model = help_command.obj
+        form_factors = getattr(operation_model, 'form_factors', None)
+        if not form_factors:
+            service_model = getattr(operation_model, 'service_model', None)
+            if service_model:
+                form_factors = getattr(service_model, 'form_factors', None)
+        if form_factors:
+            doc = help_command.doc
+            doc.style.h2('Form Factors')
+            doc.write(', '.join(form_factors))
+            doc.style.new_paragraph()
+
+    def doc_member(self, doc, member_name, member_shape, stack):
+        if member_shape.name in stack:
+            # Document the recursion once, otherwise just
+            # note the fact that it's recursive and return.
+            if stack.count(member_shape.name) > 1:
+                if member_shape.type_name == OBJECT_TYPE:
+                    doc.write('( ... recursive ... )')
+                return
+        stack.append(member_shape.name)
+        try:
+            self._do_doc_member(doc, member_name,
+                                member_shape, stack)
+        finally:
+            stack.pop()
+
+    def _do_doc_member(self, doc, member_name, member_shape, stack):
+        docs = member_shape.documentation
+        if member_name:
+            doc.write('%s -> (%s)' % (member_name, member_shape.type_name))
+        else:
+            doc.write('(%s)' % member_shape.type_name)
+        doc.style.indent()
+        doc.style.new_paragraph()
+        doc.include_doc_string(docs)
+        if member_shape.form_factors is not None:
+            self._write_form_factors(doc, member_shape.form_factors)
+        doc.style.new_paragraph()
+        member_type_name = member_shape.type_name
+        if member_type_name == OBJECT_TYPE:
+            for sub_name, sub_shape in member_shape.members.items():
+                self.doc_member(doc, sub_name, sub_shape, stack)
+        elif member_type_name == LIST_TYPE:
+            self.doc_member(doc, 'item', member_shape.member, stack)
+        elif member_type_name == MAP_TYPE:
+            self.doc_member(doc, 'key', member_shape.key, stack)
+            self.doc_member(doc, 'value', member_shape.value, stack)
+        doc.style.dedent()
+        doc.style.new_paragraph()
+
+    def _document_enums(self, argument, doc):
+        if hasattr(argument, 'argument_model'):
+            model = argument.argument_model
+            if isinstance(model, StringShape):
+                if model.enum:
+                    self._write_possible_values(doc, model.enum)
+                if model.supported_options:
+                    self._write_possible_values(doc, model.supported_options)
+
+    def _write_possible_values(self, doc, possible_values):
+        doc.style.new_paragraph()
+        doc.write('Possible values:')
+        doc.style.start_ul()
+        for value in possible_values:
+            doc.style.li('``%s``' % value)
+        doc.style.end_ul()
+
+    def _write_form_factors(self, doc, form_factors):
+        doc.style.new_paragraph()
+        doc.write('Form Factors: ')
+        doc.write(', '.join(form_factors))
+        doc.style.new_paragraph()
